// Editor methods involved in drawing to the console
package editor

import (
	"fmt"
	"strings"
	"time"
	"timshome.page/gilo/internal/gilo"
	"timshome.page/gilo/internal/terminal"
)

// ----------------------------------------------------------------------------
// !Editor Methods
// ----------------------------------------------------------------------------

func (e *editor) RefreshScreen() {
	e.scroll()

	ab := gilo.NewBuffer()

	ab.Append(terminal.HideCursor)
	ab.Append(terminal.ResetCursor)

	e.drawRows(ab)
	e.drawStatusBar(ab)
	e.drawMessageBar(ab)

	ab.Append(terminal.MoveCursor(e.renderX-e.offset.X, e.cursor.Y-e.offset.Y))

	ab.Append(terminal.ShowCursor)

	terminal.Write(ab.ToString())
}

func (e *editor) scroll() {
	e.renderX = 0

	if e.cursor.Y < e.document.RowCount() {
		e.renderX = e.document.Row(e.cursor.Y).CursorXToRenderX(e.cursor.X)
	}

	if e.cursor.Y < e.offset.Y {
		e.offset.Y = e.cursor.Y
	}

	if e.cursor.Y >= e.offset.Y+e.screen.Rows {
		e.offset.Y = e.cursor.Y - e.screen.Rows + 1
	}

	if e.renderX < e.offset.X {
		e.offset.X = e.renderX
	}

	if e.renderX >= e.offset.X+e.screen.Cols {
		e.offset.X = e.renderX - e.screen.Cols
	}
}

func (e *editor) drawRows(ab *gilo.Buffer) {
	for y := 0; y < e.screen.Rows; y++ {
		fileRow := y + e.offset.Y

		if fileRow >= e.document.RowCount() {
			e.drawPlaceholderRow(y, ab)
		} else {
			rawRow := e.document.Row(fileRow)

			// If the column offset is greater than the length of the row,
			// just display an empty row
			if e.offset.X > rawRow.Size() {
				ab.Append("")

				continue
			}

			rowLen := e.document.Row(fileRow).RenderSize() - e.offset.X
			outputRow := truncate(e.document.Row(fileRow).Render(e.offset), rowLen)
			ab.Append(outputRow)
		}

		ab.AppendLn(terminal.ClearLine)
	}
}

func (e *editor) drawPlaceholderRow(y int, ab *gilo.Buffer) {
	if e.document.RowCount() == 0 && y == e.screen.Rows/3 {
		welcome := fmt.Sprintf("Gilo editor -- version %s", gilo.Version)
		if len(welcome) > e.screen.Cols {
			welcome = truncate(welcome, e.screen.Cols)
		}

		padding := (e.screen.Cols - len(welcome)) / 2
		if padding > 0 {
			ab.AppendRune('~')
			padding--
		}

		for padding > 0 {
			padding--
			ab.AppendRune(' ')
		}

		ab.Append(welcome)
	} else {
		ab.AppendRune('~')
	}
}

func (e *editor) drawStatusBar(ab *gilo.Buffer) {
	cols := e.screen.Cols

	ab.Append(terminal.InvertColor)

	fileName := "[No Name]"
	if e.document.Filename != "" {
		fileName = e.document.Filename
	}
	modified := ""
	if e.document.IsDirty() {
		modified = "(modified)"
	}

	leftStatus := fmt.Sprintf("%.20s - %d lines %s", fileName, e.document.RowCount(), modified)
	length := len(leftStatus)
	if length > cols {
		leftStatus = truncate(leftStatus, cols)

		ab.Append(leftStatus)
		ab.Append(terminal.ResetColor)

		return
	}

	rightStatus := fmt.Sprintf("%d/%d", e.cursor.Y+1, e.document.RowCount())
	rlength := len(rightStatus)
	statusLength := length + rlength

	if statusLength <= cols {
		paddingLength := cols - statusLength
		padding := strings.Repeat(" ", paddingLength)

		ab.Append(leftStatus)
		ab.Append(padding)
		ab.Append(rightStatus)

		ab.Append(terminal.ResetColor)

		return
	}

	ab.Append(leftStatus)

	// Pad the rest of the status line
	padding := strings.Repeat(" ", cols-length)
	ab.Append(padding)

	ab.Append(terminal.ResetColor)
}

func (e *editor) drawMessageBar(ab *gilo.Buffer) {
	ab.Append("\r\n")
	ab.Append(terminal.ClearLine)

	msg := truncate(e.status.message, e.screen.Cols)
	if len(msg) > 0 && time.Since(e.status.created).Seconds() < 5.0 {
		ab.Append(msg)
	}
}
